package database;

import global.GlobalData;
import tratamientoCadenas.Cadena;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.StringTokenizer;

/**
 * Clase para rellenar las tablas relativas a los datos geograficos de la BD de GoodWave
 * */
public class RellenarGeografiaEspanola {

	/**
	 * Inserta en la BD los datos de pais, autonomia, provincia y municipio contenidos en 
	 * los ficheros pasados como argumentos
	 * @param args[] tendr los siguientes campos:
	 * 	args[0] = nombre del pais
	 * 	args[1] = cdigo del pais
	 * 	args[2] = nombre del fichero de comunidades autnomas
	 * 	args[3] = nombre del fichero de provincias
	 * 	args[4] = nombre del fichero de municipios 
	 * 
	 * @pre los ficheros tendrn el formato:
	 * 
	 * 		CODLUGAR NOMBRE_LUGAR NOMBRE_LUGAR_PERTENENCIA
	 * */
	public static void main( String args[] ) throws InterruptedException {
		
		Connection conn = null;
		String linea = null;
		FileReader fr = null;
		BufferedReader entrada = null;

		//----------  CONEXION A LA BD  ----------------
		
        try {
			Class.forName("com.mysql.jdbc.Driver");
			
	        //TODO controlar acceso identificado
	        conn = DriverManager.getConnection(GlobalData.url,GlobalData.login,GlobalData.password);

	        if (conn == null)
	        {
	           System.out.println("Conexin a base de datos "+GlobalData.url+" ... falhida");
	        }
	        
			//----------  INSERCION DEL PAIS  ----------------
			
	        try{
	            Statement st = conn.createStatement();		             
	            
	            st.executeUpdate("INSERT INTO geografia (LugarId, Nombre, Nivel)" +
	           		 "VALUES (" + Integer.parseInt(args[1]) + ",'" + args[0] + "','pais')");
	            //no es necesaria insercin en jerarqua por estar los pases en el nivel ms alto
	            //considerado de jerarqua
	          }
	          catch(SQLException se){
	  	        if (se.getMessage().contains("Duplicate entry")){
		       	 	System.out.println("El pas \"" + args[0] + "\" con codigo \"" + args[1] + "\" ya se encuentra el la Base de Datos");
		        }else{
		        	System.out.println(se);
		        }
	            System.out.println("Se prosigue a introducir los datos en la tabla relativos al pas ya existente");
	        }
	        
	        //---------  INSERCION DE COMUNIDADES AUTONOMAS  --------------

			fr = new FileReader(args[2]);	//args[2] -> fichero de autonomias
			entrada = new BufferedReader(fr);
			
	        while ((linea = entrada.readLine()) != null && (linea.length()>1))
			{
	        	introducirLugarEnBD(conn, linea, "autonomia");
		    }
	        fr.close();  
	        
	        //---------  INSERCION DE PROVINCIAS  --------------

			fr = new FileReader(args[3]);	//args[3] -> fichero de provincias
			entrada = new BufferedReader(fr);
			
	        while ((linea = entrada.readLine()) != null && (linea.length()>1))
			{
	        	introducirLugarEnBD(conn, linea, "provincia");
		    }
	        fr.close();  

	        //---------  INSERCION DE MUNICIPIOS  --------------

			fr = new FileReader(args[4]);	//args[2] -> fichero de autonomias
			entrada = new BufferedReader(fr);
			
	        while ((linea = entrada.readLine()) != null && (linea.trim().length()>1))
			{
	        	introducirLugarEnBD(conn, linea, "municipio");
		    }
	        fr.close();  
	        
	        //------- FIN CONEXION A LA BD  ----------------
	        conn.close();
	        
		} catch (ClassNotFoundException e1) {
			e1.printStackTrace();
		} catch (SQLException e) {
			System.out.println("No se ha podido conectar con la Base de Datos");
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			System.out.println("No se encuentran alguno o varios archivos de datos");
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
        
	}


	//Guarda en la BD un lugar dado a travs de una lnea "linea" con el formato
	//CodLugar NombreLugar NombreRegionALaQuePertenece
	/**
	 * Guarda en la BD un lugar dado a travs de una lnea "linea" con el formato:
	 * 
	 * 		CodLugar NombreLugar NombreRegionALaQuePertenece
	 * 
	 * @param conn Conexion a la base de datos GoodWave
	 * @param linea <b>String</b> com el formato: CodLugar NombreLugar NombreRegionALaQuePertenece
	 * @param nivel Nombre del nivel al que pertenece el lugar contenido en <i>linea</i>
	 * */
	public static void introducirLugarEnBD(Connection conn, String linea, String nivel){
		
		StringTokenizer tok = new StringTokenizer(linea, "\t");
		String codLugarTemp = tok.nextToken();
		String lugarTemp = tok.nextToken();
		String perteneceTemp = tok.nextToken();
		int codPertenece = 0;
		String pertenece = null;
		int codLugar = 0;
		String [] nombresLugar = null;
		
		Cadena cad = new Cadena();
		nombresLugar = cad.obtenerNombresLugar(perteneceTemp);
		pertenece = nombresLugar[0];
		String nivelSuperior = GlobalData.incrementarJerarquiaRegiones(nivel);
		
		//obtener de la BD el cdigo de "pertenece"
	    try{
	        Statement st = conn.createStatement();
	        
	        ResultSet res  = st.executeQuery("SELECT LugarId FROM `geografia` " + 
					  "WHERE Nombre = '" + pertenece + "' AND Nivel = '" + nivelSuperior + "'");
	        res.next();
	        codPertenece = Integer.parseInt(res.getString("LugarId"));
	        
	    
	    }catch(SQLException s){
	        System.out.println(s);}
	    
		//crear codLugar sumando "codpertenece" y "codLugarTemp"
	    codLugar = Integer.parseInt(codPertenece + codLugarTemp);
	    
	    //separar lugarTemp en los nombres pertinentes
	    nombresLugar = cad.obtenerNombresLugar(lugarTemp);
		
	    try{
	        Statement st = conn.createStatement();
	        
	        //insertar en geografia las tuplas necesarias con los diferentes nombres y "codLugar"
	        for (int j=0; j<nombresLugar.length; j++){
	        	st.executeUpdate("INSERT INTO geografia (LugarId, Nombre, Nivel)" +
	       		 "VALUES (" + codLugar + ",'" + nombresLugar[j] + "','" + nivel + "')");
	        }
	        
	        //insertar en jerarquia UNA tupla con codLugar y codpertenece
	        st.executeUpdate("INSERT INTO jerarquia (LugarId, perteneceA)" +
	       		 "VALUES (" + codLugar + "," + codPertenece + ")");
	
	      }
	      catch(SQLException se){
	        
	        if (se.getMessage().contains("Duplicate entry")){
	       	 	System.out.println("El lugar \"" + nombresLugar[0] + "\" con codigo \"" + codLugar + "\" ya se encuentra el la Base de Datos");
	        }else{
	        	System.out.println(se);
	        }
	    	System.out.println("Se prosigue a introducir el resto de datos en la tabla");
	    }
	}
}